// Copyright (c) 2011-2013, Pacific Biosciences of California, Inc.
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted (subject to the limitations in the
// disclaimer below) provided that the following conditions are met:
//
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
//  * Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following
//    disclaimer in the documentation and/or other materials provided
//    with the distribution.
//
//  * Neither the name of Pacific Biosciences nor the names of its
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
// NO EXPRESS OR IMPLIED LICENSES TO ANY PARTY'S PATENT RIGHTS ARE
// GRANTED BY THIS LICENSE. THIS SOFTWARE IS PROVIDED BY PACIFIC
// BIOSCIENCES AND ITS CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL PACIFIC BIOSCIENCES OR ITS
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
// USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

// Author: Patrick Marks and David Alexander

#pragma once

#include <boost/noncopyable.hpp>
#include <string>

// TODO(dalexander): how can we remove this include??
//  We should move all template instantiations out to another
//  header, I presume.
#include <ConsensusCore/Quiver/SimpleRecursor.hpp>
#include <ConsensusCore/Quiver/SseRecursor.hpp>
#include <ConsensusCore/Types.hpp>
#include <ConsensusCore/Mutation.hpp>

namespace ConsensusCore
{
    template<typename R>
    class MutationScorer
    {
    public:
        typedef typename R::MatrixType    MatrixType;
        typedef typename R::EvaluatorType EvaluatorType;
        typedef R                         RecursorType;

    public:
        MutationScorer(const EvaluatorType& evaluator, const R& recursor)
            throw(AlphaBetaMismatchException);

        MutationScorer(const MutationScorer& other);
        virtual ~MutationScorer();

    public:
        std::string Template() const;
        void Template(std::string tpl)
            throw(AlphaBetaMismatchException);

        float Score() const;
        float ScoreMutation(const Mutation& m) const;

    public:
        // Accessors that are handy for debugging.
        const MatrixType* Alpha() const;
        const MatrixType* Beta() const;
        const PairwiseAlignment* Alignment() const;
        const EvaluatorType* Evaluator() const;
        const int NumFlipFlops() const { return numFlipFlops_; }

    private:
        EvaluatorType* evaluator_;
        R* recursor_;
        MatrixType* alpha_;
        MatrixType* beta_;
        MatrixType* extendBuffer_;
        int numFlipFlops_;
    };

    typedef MutationScorer<SimpleQvRecursor>       SimpleQvMutationScorer;
    typedef MutationScorer<SseQvRecursor>          SseQvMutationScorer;
    typedef MutationScorer<SparseSimpleQvRecursor> SparseSimpleQvMutationScorer;
    typedef MutationScorer<SparseSseQvRecursor>    SparseSseQvMutationScorer;
    typedef MutationScorer<SparseSimpleQvSumProductRecursor> SparseSimpleQvSumProductMutationScorer;
    typedef MutationScorer<SparseSseQvSumProductRecursor>    SparseSseQvSumProductMutationScorer;
    typedef MutationScorer<SparseSseEdnaRecursor>  SparseSseEdnaMutationScorer;
}
